# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/python-ftp-svn/source/MonitorSensors.py $
# Last committed:     $Revision: 279039 $; $Author: jansink $; $Date: 2019-08-19 11:31:01 +0200 (Mon, 19 Aug 2019) $
#
import pandas as pd
import altair as alt
import sys
import getopt

from hkvportal.io.services import dataportal
from io import StringIO

from misc.Utils import Utils
from dmsLogger.DmsLogger import DmsLogger


def all_sensors(settings):

    logger = DmsLogger.set_logger(
        settings["log_folder"], settings["logger"], settings["logfile"]
    )
    logger.info("Run MonitorSensors")

    # read settings
    hkvservices = settings["hkvservices"]
    database = settings["database"]

    ftp_url = hkvservices + 'data.asmx/read2?database=' + \
        database + settings["ftp_url"]
    std_url = hkvservices + 'data.asmx/read2?database=' + \
        database + settings["std_url"]
    tld_url = hkvservices + 'data.asmx/read2?database=' + \
        database + settings["tld_url"]

    df_ftp = pd.read_csv(ftp_url, index_col=0)
    df_ftp = df_ftp.assign(Origin='ftp')

    df_std = pd.read_csv(std_url, index_col=0)
    df_std = df_std.assign(Origin='standard')

    df_tld = pd.read_csv(tld_url, index_col=0)
    df_tld = df_tld.assign(Origin='tailored')

    df = pd.concat([df_std, df_tld, df_ftp], axis=0, sort=True)

    dp = dataportal()
    dp.setDataservice(dataservice=hkvservices)

    for s in df.Sensor.unique():
        df_sensor = df[df.Sensor == s]

        single_nearest = alt.selection_single(on='mouseover', empty='none')

        title = 'Sensor {} at {}'.format(
            s, pd.datetime.today().strftime('%Y-%m-%d'))

        chart = alt.Chart(df_sensor).mark_rect().encode(
            y=alt.Y('Date:N', sort='descending'),
            x=alt.X('Locatie:N', title=None),
            fill=alt.Fill('% Received:Q',
                          scale=alt.Scale(domain=[0, .85, 1.00],
                                          range=['red', '#FFDC00', '#2ECC40']),
                          legend=alt.Legend(values=[0, 0.50, 1], title='Health', format='.0%')),
            stroke=alt.condition(single_nearest,
                                 alt.StrokeValue('black'),
                                 alt.StrokeValue('white')),
            tooltip=[
                'Locatie:N',
                'Sensor:N',
                'Files received:N',
                'Date:N',
                alt.Tooltip('% Received:Q', format='.0%'),
                alt.Tooltip('Valid files:Q', format='.0%'),
            ],
            facet=alt.Facet('Origin:N', title=title),
        ).resolve_scale(x='independent').configure_scale(bandPaddingInner=0.05, bandPaddingOuter=0.05).configure_axisX(labelAngle=-45).configure_axisY(titleAngle=0, titleAlign='left', titleY=-3, titleX=-50).configure_header(titleFontSize=16, titleAnchor='start').add_selection(single_nearest)

        # export visualizatie als een 'in-memory text stream'
        export_html = StringIO()
        chart.save(export_html, format='html')

        key = 'sensor_'+s
        logger.info("{}".format(
            hkvservices+'data.asmx/read2?database=' + database+'&key='+key+'&contentType=text/html'))

        # save data in database
        dp.setEntryDatabase(database=database,
                            key=key,
                            data=export_html.getvalue(),
                            description='Sensor ' + s)

    for l in df.Locatie.unique():
        df_locatie = df[df.Locatie == l]

        single_nearest = alt.selection_single(on='mouseover', empty='none')

        title = 'Locatie {} at {}'.format(
            l, pd.datetime.today().strftime('%Y-%m-%d'))

        chart = alt.Chart(df_locatie).mark_rect().encode(
            y=alt.Y('Date:N', sort='descending'),
            x=alt.X('Sensor:N', title=None),
            fill=alt.Fill('% Received:Q', scale=alt.Scale(domain=[0, .85,
                                                                      1.00], range=['red', '#FFDC00', '#2ECC40']),
                          legend=alt.Legend(values=[0, 0.50, 1], title='Health', format='.0%')),
            stroke=alt.condition(single_nearest, alt.StrokeValue('black'),
                                 alt.StrokeValue('white')),
            tooltip=[
                'Locatie:N',
                'Sensor:N',
                'Files received:N',
                'Date:N',
                alt.Tooltip('% Received:Q', format='.0%'),
                alt.Tooltip('Valid files:Q', format='.0%'),
            ],
            facet=alt.Facet('Origin:N', title=title),
        ).resolve_scale(x='independent').configure_scale(bandPaddingInner=0.05, bandPaddingOuter=0.05).configure_axisX(labelAngle=-45).configure_axisY(titleAngle=0, titleAlign='left', titleY=-3, titleX=-50).configure_header(titleFontSize=16, titleAnchor='start').add_selection(single_nearest)

        # export visualizatie als een 'in-memory text stream'
        export_html = StringIO()
        chart.save(export_html, format='html')

        key = 'locatie_'+l
        logger.info("{}".format(
            hkvservices+'data.asmx/read2?database=' + database+'&key='+key+'&contentType=text/html'))

        # save data in database
        dp.setEntryDatabase(database=database,
                            key=key,
                            data=export_html.getvalue(),
                            description='Locatie ' + s)

    logger.info("End of MonitorSensors")


def main(argv):
    # input argument checking
    try:
        opts, args = getopt.getopt(argv, "hs:", ["settings="])
    except getopt.GetoptError:
        print("MonitorSensors.py -s <settings_file>")
        sys.exit(2)
    for opt, arg in opts:
        if opt == "-h":
            print("MonitorSensors.py -s <settings_file>")
            sys.exit()
        elif opt in ("-s", "--settings"):
            settings_file = arg

    try:
        settings = Utils.read_monitor_sensors_settings(settings_file)
        all_sensors(settings)
    except FileNotFoundError:
        exit(1)


if __name__ == "__main__":
    main(sys.argv[1:])
